package com.itseasy.rtf;

import com.itseasy.rtf.RTFAbstractDocument;
import com.itseasy.rtf.RtfUtil;
import com.itseasy.rtf.exception.BookmarkIsNotCheckboxException;
import com.itseasy.rtf.exception.BookmarkNotFoundException;
import com.itseasy.rtf.text.Font;
import com.itseasy.rtf.text.MultiParagraphs;
import com.itseasy.rtf.text.Paragraph;
import com.itseasy.rtf.text.Text;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class RTFTemplate extends RTFAbstractDocument {
   private StringBuffer content = new StringBuffer();
   private Set fonts = new HashSet();
   private String cache;
   private File cachefile;
   private boolean activecache = false;

   public RTFTemplate(File file) throws IOException {
      this.activecache = false;
      this.cachefile = file;
      this.readFile(file);
   }

   public RTFTemplate(File file, boolean setcache) throws IOException {
      this.activecache = setcache;
      this.cachefile = file;
      this.readFile(file);
      if(setcache) {
         this.cache = this.content.toString();
      }

   }

   public void reset() throws IOException {
      this.fonts.clear();
      if(this.activecache) {
         this.content = new StringBuffer(this.cache);
      } else {
         this.readFile(this.cachefile);
      }

   }

   private void readFile(File file) throws IOException {
      char[] buffer = new char[1024];
      FileReader in = new FileReader(file);

      while(in.read(buffer) >= 0) {
         this.content.append(buffer);
      }

      in.close();
   }

   public boolean existBookmark(String name) {
      int pos = this.content.toString().indexOf("\\bkmkstart " + name + "}");
      return pos >= 0;
   }

   private RTFTemplate.BookmarkInfo searchBookmark(String name) throws BookmarkNotFoundException {
      RTFTemplate.BookmarkInfo result = null;
      String rtfdoc = this.content.toString();
      int begin = rtfdoc.indexOf("\\bkmkstart " + name + "}");
      if(begin < 0) {
         begin = rtfdoc.indexOf("\\bkmkstart " + name + '\r');
      }

      if(begin < 0) {
         throw new BookmarkNotFoundException(name, "Bookmark \'" + name + "\' not found!");
      } else {
         int contbegin;
         for(contbegin = rtfdoc.indexOf("}", begin); rtfdoc.charAt(contbegin) == 125; ++contbegin) {
            ;
         }

         int end = rtfdoc.indexOf("\\bkmkend " + name + "}", contbegin);
         if(end < 0) {
            end = rtfdoc.indexOf("\\bkmkend " + name + '\r', contbegin);
         }

         if(end < 0) {
            throw new BookmarkNotFoundException(name, "Endtag of bookmark \'" + name + "\' not found!");
         } else {
            int contend;
            for(contend = rtfdoc.lastIndexOf("{", end); rtfdoc.charAt(contend) == 123; --contend) {
               ;
            }

            ++contend;
            result = new RTFTemplate.BookmarkInfo(name, contbegin, contend, rtfdoc.substring(contbegin, contend));
            return result;
         }
      }
   }

   private void setBookmarkRawContent(String bookmark, String text) throws BookmarkNotFoundException {
      RTFTemplate.BookmarkInfo bmi = this.searchBookmark(bookmark);
      if(bmi.isField()) {
         String bkmkcontent = bmi.getRawContent();
         int contpos = bkmkcontent.indexOf("\\fldrslt");
         int inspos;
         if(contpos >= 0) {
            inspos = bkmkcontent.lastIndexOf("{", contpos);
            int end = RtfUtil.getPositionOfTagEnd(bkmkcontent, inspos);
            this.content.replace(bmi.getContentBegin() + inspos, bmi.getContentBegin() + end, "{\\fldrslt " + text + "}");
         } else {
            inspos = bkmkcontent.lastIndexOf("}");
            this.content.insert(bmi.getContentBegin() + inspos, "{\\fldrslt " + text + "}");
         }
      } else {
         this.content.replace(bmi.getContentBegin(), bmi.getContentEnd(), text);
      }

   }

   public void setBookmarkContent(String bookmark, String text) throws BookmarkNotFoundException {
      this.setBookmarkRawContent(bookmark, RtfUtil.getRTFString(text));
   }

   public void setBookmarkContent(String bookmark, Text textparts) throws BookmarkNotFoundException {
      this.setBookmarkRawContent(bookmark, textparts.getRtfContent());
   }

   public void setBookmarkContent(String bookmark, Paragraph text, boolean insideTable) throws BookmarkNotFoundException {
      Set fs = text.getAllFonts();
      if(fs != null) {
         this.fonts.addAll(fs);
      }

      this.setBookmarkRawContent(bookmark, text.getRtfContent(insideTable));
   }

   public void setBookmarkContent(String bookmark, Paragraph text) throws BookmarkNotFoundException {
      this.setBookmarkContent(bookmark, text, false);
   }

   public void setBookmarkContent(String bookmark, MultiParagraphs text, boolean insideTable) throws BookmarkNotFoundException {
      Set fs = text.getAllFonts();
      if(fs != null) {
         this.fonts.addAll(fs);
      }

      this.setBookmarkRawContent(bookmark, text.getRtfContent(insideTable));
   }

   public void setBookmarkCheckbox(String bookmark, boolean value) throws BookmarkNotFoundException, BookmarkIsNotCheckboxException {
      RTFTemplate.BookmarkInfo bmi = this.searchBookmark(bookmark);
      String cbresult = value?"1":"25";
      if(bmi.isField()) {
         if(bmi.getFieldtype() != 1) {
            throw new BookmarkIsNotCheckboxException(bookmark);
         } else {
            int posres = bmi.getRawContent().indexOf("\\ffres");
            int postype;
            if(posres >= 0) {
               postype = 0;

               int countstart;
               for(countstart = bmi.getContentBegin() + posres + 6; Character.isDigit(this.content.charAt(countstart + postype)); ++postype) {
                  ;
               }

               this.content.replace(countstart, countstart + postype, cbresult);
            } else {
               postype = bmi.getRawContent().indexOf("\\fftype");
               if(postype < 0) {
                  throw new BookmarkNotFoundException(bookmark, "Typedefinition not found!");
               }

               this.content.insert(bmi.getContentBegin() + postype + 8, "\\ffres " + cbresult);
            }

         }
      } else {
         throw new BookmarkIsNotCheckboxException(bookmark);
      }
   }

   protected String getDocumentAsString() {
      if(this.fonts.size() > 0) {
         this.setFonttable();
      }

      return this.content.toString();
   }

   protected void setFonttable() {
      if(this.fonts.size() > 0) {
         StringBuffer tbl = new StringBuffer();
         Iterator fsit = this.fonts.iterator();

         while(fsit.hasNext()) {
            Font pos = (Font)fsit.next();
            if(pos != null) {
               tbl.append("{" + pos.getRtfContent() + "}");
            }
         }

         int pos1 = this.content.indexOf("\\fonttbl", 0);
         if(pos1 >= 0) {
            int insertpos = RtfUtil.getPositionOfTagEnd(this.content.toString(), pos1);
            this.content.insert(insertpos - 1, tbl.toString());
         } else {
            pos1 = this.content.indexOf("{", 1);
            this.content.append("\\fonttbl");
            this.content.insert(pos1, tbl.toString());
         }
      }

   }

   public List getBookmarkNames() {
      ArrayList allbm = new ArrayList();

      for(int pos = 0; (pos = this.content.indexOf("\\bkmkstart ", pos)) >= 0; ++pos) {
         int end = this.content.indexOf("}", pos);
         allbm.add(this.content.substring(pos + 11, end));
      }

      return allbm;
   }

   protected class BookmarkInfo {
      public static final int TYPE_NIL = -1;
      public static final int TYPE_TEXT = 0;
      public static final int TYPE_CHECKBOX = 1;
      public static final int TYPE_LISTBOX = 2;
      private String name;
      private int contentbegin;
      private int contentend;
      private boolean field;
      private String rawcontent;

      public BookmarkInfo(String name, int begin, int end, String rawcontent) {
         this.name = name;
         this.contentbegin = begin;
         this.contentend = end;
         this.rawcontent = rawcontent;
         if(rawcontent != null && rawcontent.indexOf("\\formfield") >= 0) {
            this.field = true;
         } else {
            this.field = false;
         }

      }

      public int getContentBegin() {
         return this.contentbegin;
      }

      public int getContentEnd() {
         return this.contentend;
      }

      public boolean isField() {
         return this.field;
      }

      public boolean isFieldReadOnly() {
         boolean result = false;
         if(this.isField()) {
            if(this.rawcontent != null && this.rawcontent.indexOf("\\fldlock") >= 0) {
               result = true;
            } else if(this.rawcontent != null && this.rawcontent.indexOf("\\ffprot1") >= 0) {
               result = true;
            } else {
               result = false;
            }
         }

         return result;
      }

      public int getFieldMaxLength() {
         int result = -1;
         if(this.isField() && this.rawcontent != null) {
            int pos = this.rawcontent.indexOf("\\ffmaxlen");
            if(pos >= 0) {
               result = RtfUtil.getIntegerFromText(this.rawcontent, pos + 9);
            }
         }

         return result;
      }

      public int getFieldtype() {
         byte result = -1;
         if(this.field) {
            int posfd = this.rawcontent.indexOf("\\fftype");
            if(posfd >= 0) {
               switch(this.rawcontent.charAt(posfd + 7)) {
               case '0':
                  result = 0;
                  break;
               case '1':
                  result = 1;
                  break;
               case '2':
                  result = 2;
                  break;
               default:
                  result = -1;
               }
            }
         }

         return result;
      }

      public String getRawContent() {
         return this.rawcontent;
      }
   }
}
